# -*- coding: utf-8 -*-
"""
Gender bias, citizen participation, and AI
Natalia Pecorari and Jose Cuesta, 2024
Using Latinobarometro 2023 data
"""

### Decision Trees ###
# 1. Decision tree Hyperparameter optimization using Grid search
# 2. Decision trees generation 
# 3. Introducing model variations
# 4. Counterfactuals
# 5. Robustness checks

# Imports
import os
import pandas as pd
import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt
from matplotlib import pyplot
from sklearn.tree import DecisionTreeClassifier, export_graphviz
from sklearn.model_selection import train_test_split, GridSearchCV, KFold
from sklearn.inspection import PartialDependenceDisplay
from sklearn.pipeline import Pipeline
from sklearn import metrics, tree
import graphviz
from sklearn.tree import export_text, _tree
import difflib
import matplotlib
from contextlib import contextmanager
import sklearn


outputs = "C:/WBG/github/255/v1/DEC Reproducibility Verification - WB Group/Outputs"

# Print scikit-learn version
print('The scikit-learn version is {}.'.format(sklearn.__version__))

# Load data
dtafile = "C:/WBG/github/255/v1/DEC Reproducibility Verification - WB Group/DataOut/Latinobarometro_2023_no_missing.dta"
df = pd.read_stata(dtafile)
df.head()

# Convert 'education', 'social_class' and male from category to int
df['education'] = df['education'].astype(int)
df['social_class'] = df['social_class'].astype(int)
df['male'] = df['male'].astype(int)

# Ensure 'age' is numeric
df['age'] = df['age'].astype(float)

# Check data types
print(df.dtypes)

# Mean of composite gender score by country
mean_by_country = df.groupby('country', observed=False)['composite_gender_score'].mean().reset_index()
mean_by_country.columns = ['Country', 'Mean Composite Gender Score']
mean_by_country

### 1. Decision Trees Hyperparameter optimization using Grid search ###

def grid_search_decision_tree(X, y):
    dec_tree = tree.DecisionTreeClassifier()
    pipe = Pipeline(steps=[('dec_tree', dec_tree)])
    criterion = ['gini', 'entropy']
    max_depth = [2, 3, 4, 5, 6, 8, 10, 12]
    parameters = dict(dec_tree__criterion=criterion, dec_tree__max_depth=max_depth)
    cv = KFold(n_splits=5, shuffle=True, random_state=42)
    clf_GS = GridSearchCV(pipe, parameters, cv=cv, return_train_score=False)
    clf_GS.fit(X, y)
    
    # Get the results as a DataFrame
    results_df = pd.DataFrame(clf_GS.cv_results_)
    
    # Extract relevant columns
    results_df = results_df[['param_dec_tree__criterion', 'param_dec_tree__max_depth', 'mean_test_score']]
    
    # Rename columns for clarity
    results_df.rename(columns={'param_dec_tree__criterion': 'Criterion', 
                               'param_dec_tree__max_depth': 'Max Depth', 
                               'mean_test_score': 'Mean Test Score'}, inplace=True)
    
    # Sort the DataFrame by Mean Test Score in descending order
    results_df = results_df.sort_values(by='Mean Test Score', ascending=False).reset_index(drop=True)
    
    # Print the best parameters
    print('Best Criterion:', clf_GS.best_estimator_.get_params()['dec_tree__criterion'])
    print('Best max_depth:', clf_GS.best_estimator_.get_params()['dec_tree__max_depth'])
    print(clf_GS.best_estimator_.get_params()['dec_tree'])
    
    # Print the sorted results DataFrame
    print(results_df)
    
    return clf_GS

# Baseline model - determining optimal depth and criterion
X = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y = df['civic_part']
clf_GS = grid_search_decision_tree(X, y)

### 2. Decision trees generation ###

def train_and_evaluate_decision_tree(X, y, max_depth, criterion, weights):
    # Split the dataset into training and testing sets
    X_train, X_test, y_train, y_test, weights_train, weights_test = train_test_split(X, y, weights, test_size=0.2, random_state=1)
    
    # Train the Decision Tree Classifier on the training data
    clf = DecisionTreeClassifier(random_state=1, max_depth=max_depth, criterion=criterion)
    clf.fit(X_train, y_train, sample_weight=weights_train)  # Use the sliced weights for training
    
    # Visualize the trained decision tree
    data = export_graphviz(clf, out_file=None, feature_names=X.columns, class_names=['0', '1'], filled=True, rounded=True, special_characters=True)
    graph = graphviz.Source(data)
    #display(graph)
    
    # Make predictions on the test set
    y_pred = clf.predict(X_test)
    
    # Calculate and print the accuracy
    print("Accuracy:", metrics.accuracy_score(y_test, y_pred))
    
    # Calculate feature importance scores
    feature_scores = pd.Series(clf.feature_importances_, index=X.columns).sort_values(ascending=False)
    
    # Plot the feature importance scores
    plt.figure(figsize=(10, 6))
    sns.barplot(x=feature_scores, y=feature_scores.index, palette="vlag")
    plt.xlabel('Feature Importance Score')
    plt.ylabel('Variables')
    plt.title("Important Variables")
    plt.legend()
    plt.show()
    
    return clf

# Baseline model: Work for a community problem - Decision tree and feature importance
# This generates figure 2 and 3)
X = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y = df['civic_part']
weights = df['wt']
clf = train_and_evaluate_decision_tree(X, y, max_depth=4, criterion='gini', weights=weights)
plt.savefig(os.path.join(outputs, "figure3.png"))

# Partial Dependence Plots for education (figure 5)

features3 = ['education']
fig, ax = plt.subplots(figsize=(10, 6))
PartialDependenceDisplay.from_estimator(clf, X, features3, ax=ax, grid_resolution=50)
plt.title('Partial Dependence Plot for education')
plt.savefig(os.path.join(outputs, "figure5.png"))
plt.show()

# Interaction plot for composite gender score and education (figure 6)
plt.figure(figsize=(10, 6))
sns.lmplot(x='composite_gender_score', y='civic_part', hue='education', data=df, 
           aspect=2, height=6, palette='viridis')

plt.title('Interaction Plot of Composite Gender Score and Education on Working for a community problem')
plt.xlabel('Composite Gender Score')
plt.ylabel('Work for a community problem')

# Remove plt.legend() since sns.lmplot already handles legends
plt.savefig(os.path.join(outputs, "figure6.png"))
plt.show()


##### 3. Introducing model variations #####

# Removing gender indicator
X_v1 = df[['age', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_v1 = df['civic_part']
weights_v1 = df['wt']
clf_GS = grid_search_decision_tree(X_v1, y_v1)
clf = train_and_evaluate_decision_tree(X_v1, y_v1, max_depth=4, criterion='gini', weights=weights_v1)

# Two separate trees for women and for men

# Tree for women

# Filter the dataset to keep only observations where male == 0 and reset the index
df_women = df[df['male'] == 0]
df_women.reset_index(drop=True, inplace=True)

X_women = df_women[['age', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_women = df_women['civic_part']
weights_women = df_women['wt']
clf_GS = grid_search_decision_tree(X_women, y_women)
clf = train_and_evaluate_decision_tree(X_women, y_women, max_depth=4, criterion='entropy', weights=weights_women)

# Tree for men

# Filter the dataset to keep only observations where male == 0 and reset the index
df_men = df[df['male'] == 1]
df_men.reset_index(drop=True, inplace=True)

X_men = df_men[['age', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_men = df_men['civic_part']
weights_men = df_men['wt']
clf_GS = grid_search_decision_tree(X_men, y_men)
clf = train_and_evaluate_decision_tree(X_men, y_men, max_depth=4, criterion='gini', weights=weights_men)

# Removing gender indicator and variables highly correlated

# Plot Heatmap:
variables = [ 'age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house',
             'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak',
             'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 
             't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']

# Select only the specified variables
df_subset = df[variables]

# Calculate the correlation matrix
corr_matrix = df_subset.corr()

# Extract correlations with the 'male' variable
male_corr = corr_matrix[['male']].sort_values(by='male', ascending=False)

# Create a heatmap focused on 'male'
#plt.figure(figsize=(8, 10))
#sns.heatmap(male_corr, annot=True, cmap='coolwarm', fmt='.2f', linewidths=0.5)
#plt.title('Correlation with Male')
#plt.show()

# Removing gender and employed variables
X_v3 = df[['age', 'education', 'social_class', 'subsidy', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_v3 = df['civic_part']
weights_v3 = df['wt']
clf_GS = grid_search_decision_tree(X_v3, y_v3)
clf = train_and_evaluate_decision_tree(X_v3, y_v3, max_depth=4, criterion='gini', weights=weights_v3)

### Inverting the male variable

df['inverted_male'] = 1 - df['male']

X_inv = df[['age', 'inverted_male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_inv = df['civic_part']
weights_inv = df['wt']
clf_GS = grid_search_decision_tree(X_inv, y_inv)
clf = train_and_evaluate_decision_tree(X_inv, y_inv, max_depth=4, criterion='gini', weights=weights_inv)

###### 4. Introducing counterfactuals #######

# Define the gender perception score
gender_feature = 'composite_gender_score'

# Original scores
original_scores = df[gender_feature]

# Strategy 1: Shift the distribution to the left
def shift_distribution(scores, shift_amount, lower_bound):
    shifted_scores = scores - shift_amount
    return np.clip(shifted_scores, lower_bound, np.max(scores))

shift_amount = 0.25  # Adjust this value as needed
lower_bound = 0

shifted_scores = shift_distribution(original_scores, shift_amount, lower_bound)

# Strategy 2: Scale down the values
def scale_down(scores, adjustment_factor):
    return np.clip(scores * adjustment_factor, 0, 1)

original_mean = df['composite_gender_score'].mean()
target_mean = original_mean * 0.5  # Targeting a society with half the machismo

# Calculate the adjustment factor
adjustment_factor = target_mean / original_mean

# Get the original scores
original_scores = df['composite_gender_score'].values

# Scale down the scores
scaled_scores = scale_down(original_scores, adjustment_factor)

# Strategy 3: Scale up the values
def scale_up(scores, adjustment_factor):
    return np.clip(scores * adjustment_factor, 0, 1)

original_mean = df['composite_gender_score'].mean()
target_mean = original_mean * 2  # Targeting a society with double the machismo

# Calculate the adjustment factor
adjustment_factor = target_mean / original_mean

# Get the original scores
original_scores = df['composite_gender_score'].values

# Scale up the scores
scaled_scores_double_machismo = scale_up(original_scores, adjustment_factor)

# Add counterfactual vectors to the dataframe for further analysis
df['shifted_gender_score'] = shifted_scores
df['scaled_gender_score'] = scaled_scores
df['scaled_up_gender_score'] = scaled_scores_double_machismo


# Using counterfactual 1: scaled score (half machismo)
X_v4 = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'scaled_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_v4 = df['civic_part']
weights_v4 = df['wt']
clf_GS = grid_search_decision_tree(X_v4, y_v4)
clf = train_and_evaluate_decision_tree(X_v4, y_v4, max_depth=4, criterion='gini', weights=weights_v4)

# Using counterfactual 2: shifted score
X_v5 = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'shifted_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_v5 = df['civic_part']
weights_v5 = df['wt']
clf_GS = grid_search_decision_tree(X_v5, y_v5)
clf = train_and_evaluate_decision_tree(X_v5, y_v5, max_depth=4, criterion='gini', weights=weights_v5)

# Using counterfactual 3: scaled up score (doubled machismo)
X_v6 = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'scaled_up_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_v6 = df['civic_part']
weights_v6 = df['wt']
clf_GS = grid_search_decision_tree(X_v6, y_v6)
clf = train_and_evaluate_decision_tree(X_v6, y_v6, max_depth=4, criterion='gini', weights=weights_v6)


#### Composite gender score in success path plots ####

### Decision trees generation v2 ###
def train_and_evaluate_decision_tree_v2(X, y, max_depth, criterion, weights):
    # Split the dataset into training and testing sets
    X_train, X_test, y_train, y_test, weights_train, weights_test = train_test_split(X, y, weights, test_size=0.2, random_state=1)
    
    # Train the Decision Tree Classifier on the training data
    clf = DecisionTreeClassifier(random_state=1, max_depth=max_depth, criterion=criterion)
    clf.fit(X_train, y_train, sample_weight=weights_train)  # Use the sliced weights for training
    
    # Make predictions on the test set
    y_pred = clf.predict(X_test)
    
    # Return the trained classifier and the predictions
    return clf, y_pred

# Scaling down (figure 4 A)

# Define the scaling function
def scale_down(original_scores, factor):
    return original_scores * factor

# Define a context manager to suppress plots
@contextmanager
def suppress_plots():
    import matplotlib
    original_backend = matplotlib.get_backend()
    matplotlib.use('Agg')  # Use a non-interactive backend to suppress plotting
    plt.ioff()  # Turn off interactive mode
    yield
    matplotlib.use(original_backend)  # Restore the original backend
    plt.ion()  # Turn on interactive mode
    plt.close('all')  # Close all figures

# Function to extract composite_gender_score values from the decision tree's success path
def extract_success_path_scores(clf, feature_names):
    tree_ = clf.tree_
    scores = []
    
    def traverse(node):
        if tree_.feature[node] != _tree.TREE_UNDEFINED:  # Not a leaf node
            feature_index = tree_.feature[node]
            if feature_names[feature_index] == 'scaled_gender_score':
                scores.append(tree_.threshold[node])
            traverse(tree_.children_left[node])
            traverse(tree_.children_right[node])
    
    traverse(0)
    return scores

# Perform Grid Search to find the best hyperparameters
clf_GS = grid_search_decision_tree(X, y)

# Generate a range of scaling factors
scaling_factors = np.linspace(1, 0.01, 40)  # From 1 to 0.01 in 40 steps
success_path_scores = []
baseline_path = None

# Original feature set
X_base = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y = df['civic_part']
weights = df['wt']
original_scores = df['composite_gender_score']

# Iterate over scaling factors
for factor in scaling_factors:
    # Scale down the gender bias score
    scaled_scores = scale_down(original_scores, factor)
    df['scaled_gender_score'] = scaled_scores

    # Add the scaled score to the feature set
    X = X_base.copy()
    X['scaled_gender_score'] = df['scaled_gender_score']

    # Split the dataset into training and testing sets
    X_train, X_test, y_train, y_test, weights_train, weights_test = train_test_split(X, y, weights, test_size=0.2, random_state=1)

    # Train the decision tree using the best parameters from grid search
    best_params = clf_GS.best_params_

    # Suppress plots during training
    with suppress_plots():
        clf, y_pred = train_and_evaluate_decision_tree_v2(X_train, y_train, 
                                                           max_depth=best_params['dec_tree__max_depth'], 
                                                           criterion=best_params['dec_tree__criterion'], 
                                                           weights=weights_train)
    
    # Extract values of composite_gender_score in the success path
    success_scores = extract_success_path_scores(clf, list(X.columns))
    avg_success_score = np.mean(success_scores)  # Average value for the success path
    
    # Store the average score and scaling factor
    success_path_scores.append((factor, avg_success_score))

# Convert success_path_scores to DataFrame
success_path_scores_df = pd.DataFrame(success_path_scores, columns=['Scaling Factor', 'Average Success Path Score'])

# Plot the success path scores
plt.figure(figsize=(10, 6))
plt.plot(success_path_scores_df['Scaling Factor'], success_path_scores_df['Average Success Path Score'], marker='o')
plt.xlabel('Scaling Factor')
plt.ylabel('Average Success Path Composite Gender Score')
plt.title('Composite Gender Score in Success Path with Varying Scaling Factors')
plt.grid(True)
plt.savefig(os.path.join(outputs, "figure4A.png"))
plt.show()

# Shifting down (figure 4 B)

# Define the shift distribution function
def shift_distribution(scores, shift_amount, lower_bound):
    shifted_scores = scores - shift_amount
    return np.clip(shifted_scores, lower_bound, np.max(scores))

# Function to extract composite_gender_score values from the decision tree's success path
def extract_success_path_scores(clf, feature_names):
    tree_ = clf.tree_
    scores = []
    
    def traverse(node):
        if tree_.feature[node] != _tree.TREE_UNDEFINED:  # Not a leaf node
            feature_index = tree_.feature[node]
            if feature_names[feature_index] == 'shifted_gender_score':
                scores.append(tree_.threshold[node])
            traverse(tree_.children_left[node])
            traverse(tree_.children_right[node])
    
    traverse(0)
    return scores

# Perform Grid Search to find the best hyperparameters
clf_GS = grid_search_decision_tree(X, y)

# Generate a range of shift amounts
shift_amounts = np.linspace(0, 0.5, 40)  # From 0 to 0.5 in 40 steps
success_path_scores = []

# Original feature set
X_base = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y = df['civic_part']
weights = df['wt']
original_scores = df['composite_gender_score']

# Iterate over shift amounts
for shift_amount in shift_amounts:
    # Shift the gender bias score
    shifted_scores = shift_distribution(original_scores, shift_amount, lower_bound=0)
    df['shifted_gender_score'] = shifted_scores

    # Add the adjusted score to the feature set
    X = X_base.copy()
    X['shifted_gender_score'] = df['shifted_gender_score']

    # Split the dataset into training and testing sets
    X_train, X_test, y_train, y_test, weights_train, weights_test = train_test_split(X, y, weights, test_size=0.2, random_state=1)

    # Train the decision tree using the best parameters from grid search
    best_params = clf_GS.best_params_

    # Suppress plots during training
    with suppress_plots():
        clf, y_pred = train_and_evaluate_decision_tree_v2(X_train, y_train, 
                                                           max_depth=best_params['dec_tree__max_depth'], 
                                                           criterion=best_params['dec_tree__criterion'], 
                                                           weights=weights_train)
    
    # Extract values of composite_gender_score in the success path
    success_scores = extract_success_path_scores(clf, list(X.columns))
    avg_success_score = np.mean(success_scores)  # Average value for the success path
    
    # Store the average score and shift amount
    success_path_scores.append((shift_amount, avg_success_score))

# Convert success_path_scores to DataFrame
success_path_scores_df = pd.DataFrame(success_path_scores, columns=['Shift Amount', 'Average Success Path Score'])

# Plot the success path scores
plt.figure(figsize=(10, 6))
plt.plot(success_path_scores_df['Shift Amount'], success_path_scores_df['Average Success Path Score'], marker='o')
plt.xlabel('Shift Amount')
plt.ylabel('Average Success Path Composite Gender Score')
plt.title('Composite Gender Score in Success Path with Varying Shift Amounts')
plt.grid(True)
plt.show()

### 5. Robustness checks ###

# Signing a petition

X_s = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_s = df['petition']
weights_s = df['wt']
clf_GS_s = grid_search_decision_tree(X_s, y_s)
clf_s = train_and_evaluate_decision_tree(X_s, y_s, max_depth=5, criterion='entropy', weights=weights_s)

# Attending demonstrations

X_a = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'composite_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_a = df['demons']
weights_a = df['wt']
clf_GS_a = grid_search_decision_tree(X_a, y_a)
clf_a = train_and_evaluate_decision_tree(X_a, y_a, max_depth=4, criterion='entropy', weights=weights_a)

# Baseline model with all gender variables (instead of a composite score)

X_r = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'women_3', 'women_4', 'women_5', 'women_6', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_r = df['civic_part']
weights_r = df['wt']
clf_GS_r = grid_search_decision_tree(X_r, y_r)
clf_r = train_and_evaluate_decision_tree(X_r, y_r, max_depth=4, criterion='gini', weights=weights_r)

# Baseline model with an indicator variable for high machismo levels (instead of a composite score)

X_t = df[['age', 'male', 'education', 'social_class', 'subsidy', 'employed', 'save_money', 'own_house', 'internet', 'sewage', 'int_politics', 'political_spectrum', 'free_join', 'free_speak', 'life_satisfaction', 'victim_crime', 'enough_food', 'high_gender_score', 't_government', 't_police', 't_parliament', 't_courts', 't_elections', 't_people']]
y_t = df['civic_part']
weights_t = df['wt']
clf_GS_t = grid_search_decision_tree(X_t, y_t)
clf_t = train_and_evaluate_decision_tree(X_t, y_t, max_depth=4, criterion='gini', weights=weights_t)
